@@ -41,7 +41,7 @@ class Detection_Routine(StarFinderBase):
     def __init__(self,  sig_src=5, sig_sky=3, fwhm=1,
                         sharplo=0.2, sharphi=1, roundlo=-1, roundhi=1,
                         wcs=None, match_threshold=1.0, verbose=0,
-                        boxsize=2, filtersize=3):
+                        bgd2d=1, boxsize=2, filtersize=3):
         self.sig_src=sig_src
         self.sig_sky=sig_sky
         self.fwhm = fwhm
@@ -55,6 +55,7 @@ class Detection_Routine(StarFinderBase):
         self.catalogue=None
         self.verbose=verbose
 
+        self.bgd2d=bgd2d
         self.boxsize=boxsize
         self.filtersize=filtersize
 
@@ -115,8 +116,9 @@ class Detection_Routine(StarFinderBase):
         self.match(self.catalogue, self.detect(data, SExtractorBackground(sigma_clip=sigma_clip)))
         if self.verbose: printf("-> [SExTr] pass: %d sources\n"%len(self.catalogue))
 
-        self.match(self.catalogue, self.detect(data, self._bkg2d))
-        if self.verbose: printf("-> [BGD2D] pass: %d sources\n"%len(self.catalogue))
+        if self.bgd2d:
+            self.match(self.catalogue, self.detect(data, self._bkg2d))
+            if self.verbose: printf("-> [BGD2D] pass: %d sources\n"%len(self.catalogue))
 
         ## Now with xycoords DAOStarfinder will refit the sharp and round values at the detected locations
         self.catalogue=self.detect(data, xycoords=np.array([self.catalogue["xcentroid"],self.catalogue["ycentroid"]]).T)#, clean=0)
@@ -135,10 +137,12 @@ class APPhot_Routine():
     Aperture photometry called by starbug
     Given photometry radius, sky annuli radii rad_inner rad_outer
     """
-    def __init__(self, radius, sky_in, sky_out, verbose=0):
+    def __init__(self, radius, sky_in, sky_out, encircled_energy=50, fit_radius=1, verbose=0):
         self.radius=radius
         self.sky_in=sky_in
         self.sky_out=sky_out
+        self.encircled_energy=encircled_energy
+        self.fit_radius=fit_radius
         self.catalogue=Table(None)#, names=["ap_flux_r%d"%n for n in range(len(radii))]+["sky_median"])
 
         self.verbose=verbose
@@ -147,62 +151,59 @@ class APPhot_Routine():
     def __call__(self, detections, image, **kwargs):
         return self.run(detections, image, **kwargs)
 
-    def run(self, detections, image, apcorr=1.0, sig_sky=3):
+    #def run(self, detections, image, apcorr=1.0, sig_sky=3):
+    def run(self, detections, image, error=1, dqflags=None, apcorr=1.0, sig_sky=3):
         """
         Forced aperture photometry on a list of detections
         detections are a astropy.table.Table with columns xcentroid ycentroid or x_0 y_0
         This will add extra columns into this table ap_flux ap_sky
+        INPUT:  detections - Astropy.Table containing source list
+                image       -2D image array to run photometry on
+                error       -2D image array OR scalar to act as photometric error
+                dqflags     -2D array of data quality flags where appropriate
+
+        RETURN: Photometry catalogue
         """
-        try:
+        if len( set(("xcentroid","ycentroid")) & set(detections.colnames))==2:
             pos=[(line["xcentroid"],line["ycentroid"]) for line in detections]
-        except:
+        elif len( set(("x_0","y_0")) & set(detections.colnames))==2:
             pos=[(line["x_0"],line["y_0"]) for line in detections]
+        else:
+            perror("Cannot identify position in detection catalogue (x_0/xcentroid)\n");
+            return None
 
-        area=1.0 if image[4].name!="AREA" else image["AREA"].data
-        mask= image["DQ"].data &( DQ_DO_NOT_USE| DQ_SATURATED | DQ_JUMP_DET)
-
-        data= image["SCI"].data * area
-        data[mask]=np.nan
-        data[data<=0]=np.nan
-        error=np.sqrt(data)
+        mask=np.isnan(image)
 
         apertures=CircularAperture(pos,self.radius)
         annulus_aperture=CircularAnnulus(pos, r_in=self.sky_in, r_out=self.sky_out)
-        phot=aperture_photometry(data, apertures, error=error, mask=mask)
+        phot=aperture_photometry(image, apertures, error=error, mask=mask)
         
         self.catalogue=Table(np.full((len(pos),3),np.nan),names=("flux","eflux","sky"))
-        bkg_stats=np.full((len(annulus_aperture),2), np.nan) # (mode, stdev) 
 
         self.log("-> calculating photometric errors\n")
-        load=loading(len(apertures),"error?",res=10)
         for i,mask in enumerate(annulus_aperture.to_mask(method="center")):
-            dat=mask.multiply(data)
+            dat=mask.multiply(image)
             dat=sigma_clip(dat[dat>0 & np.isfinite(dat)], sigma=sig_sky)
-            bkg_stats[i]=sigma_clipped_stats(dat)[1:]
-            #bkg_stats[i]=( mode(dat)[0], np.nanstd(dat)) ##Take the mode to estimate background level
+            if len(dat): ##sometimes all the surrounding pixels are nan OR above SIGSKY value??
+                self.catalogue["sky"][i]=mode(dat)[0]
 
 
-        err=np.full( (len(phot),3), np.nan)
-        err[:,0]=phot["aperture_sum_err"].value      ##POISSON ERR
-        err[:,1]=apertures.area * bkg_stats[:,1]**2  ##SKY_ERR
-        err[:,2]=(apertures.area**2) * (bkg_stats[:,1]**2) / annulus_aperture.area ##MEANSKY_ERR
-        self.catalogue["eflux"]=np.sqrt(np.nansum(err,axis=1))
-
-        self.catalogue["sky"]=bkg_stats[:,0]
+        self.catalogue["eflux"]=phot["aperture_sum_err"]
         self.catalogue["flux"]=apcorr*(phot["aperture_sum"] - (self.catalogue["sky"]*apertures.area))
 
-        if "DQ" in extnames(image):
+        col=Column(np.full(len(apertures),SRC_GOOD), dtype=np.uint32, name="flag")
+        if dqflags is not None:
             self.log("-> flagging unlikely sources\n")
-            col=Column(np.full(len(apertures),SRC_GOOD), dtype=np.uint32, name="flag")
             for i, mask in enumerate(apertures.to_mask(method="center")):
-                dat=np.array(mask.multiply(image["DQ"].data),np.uint32)
+                dat=np.array(mask.multiply(dqflags),np.uint32)
                 if np.sum( dat & (DQ_DO_NOT_USE|DQ_SATURATED)): col[i]|=SRC_BAD
                 if np.sum( dat & DQ_JUMP_DET): col[i]|=SRC_JMP
-            self.catalogue.add_column(col)
-
+        self.catalogue.add_column(col)
         return self.catalogue
 
-    def calc_apcorr(self, filter, radius, table_fname=None):
+
+    @staticmethod
+    def calc_apcorr(filter, radius, table_fname=None, verbose=0):
         """
         Using CRDS apcorr table, fit a curve to the radius vs apcorr
         columns and then return aporr to respective input radius
@@ -214,13 +215,32 @@ class APPhot_Routine():
         fn=lambda x,a,b,c: a*np.exp(-b*x)+c
         popt,_=curve_fit(fn, t_apcorr["radius"], t_apcorr["apcorr"])
         apcorr= fn( radius,*popt)
-        self.log("-> estimating aperture correction: %.3g..\n"%apcorr)
+        if verbose: printf("-> estimating aperture correction: %.3g..\n"%apcorr)
 
         popt,_=curve_fit(fn,t_apcorr["radius"],t_apcorr["eefraction"])
         eefrac= fn(radius,*popt)
-        self.log("-> effective encircled energy fraction: %.3g\n"%eefrac)
+        if verbose: printf("-> effective encircled energy fraction: %.3g\n"%eefrac)
         return apcorr
 
+
+    @staticmethod
+    def apcorr_from_encenergy(filter, encircled_energy, table_fname=None, verbose=0):
+        """
+        Rather than fitting radius to the APCORR CRDS, use the closes Encircled energy value
+        """
+        if not table_fname: return None
+        tmp=Table.read(table_fname, format="fits")
+        t_apcorr=tmp[tmp["filter"]==filter]
+
+        line=t_apcorr[(np.abs(t_apcorr["eefraction"]-encircled_energy)).argmin()]
+        if verbose:
+            printf("Best matching encircled energy %.1f, with radius %g pixels\n"%(line["eefraction"],line["radius"]))
+            printf("Using aperture correction: %f\n"%line["apcorr"])
+
+        return line["apcorr"], line["radius"]
+
+
+
     def log(self,msg):
         if self.verbose: 
             printf(msg)
@@ -257,19 +277,36 @@ class _fitmodel(LevMarLSQFitter):
 
 class BackGround_Estimate_Routine(BackgroundBase):
     bgd=None
-    sourcelist=[]
-    def __init__(self, sourcelist, boxsize=2, fwhm=2, verbose=0):#mask_r0=7, mask_r1=9
+    sourcelist=None
+    def __init__(self, sourcelist, boxsize=2, fwhm=2, verbose=0, bgd=None):#mask_r0=7, mask_r1=9
         self.sourcelist=sourcelist
         self.boxsize=boxsize
         self.fwhm=fwhm
         self.verbose=verbose
+        self.bgd=bgd
+
+    def calc_peaks(self,im):
+        """
+        Determine peak pixel value for each source in xy
+        """
+        x=self.sourcelist["xcentroid"]
+        y=self.sourcelist["ycentroid"]
+        apertures=CircularAperture(np.array((x,y)).T,2).to_mask()
+        peaks=np.full(len(x),np.nan)
+        for i,mask in enumerate(apertures):
+            peaks[i]=np.nanmax( mask.multiply(im) )
+        return peaks
 
     def __call__(self, data):
+        if self.sourcelist is None: return self.bgd
         _data=np.copy(data)
         X,Y=np.ogrid[:data.shape[1], :data.shape[0]]
-        rlist=np.sqrt(self.sourcelist["peak"]**0.7)*self.fwhm
+        peaks=self.calc_peaks(data)
+
+        rlist=np.sqrt(peaks**0.7)*self.fwhm/1.5
         D=50
         load=loading(len(self.sourcelist), msg="masking sources")
+        #fp=open("out.reg","w")
         for r,src in zip(rlist,self.sourcelist):
 
             rin=1.5*r
@@ -296,10 +333,11 @@ class BackGround_Estimate_Routine(BackgroundBase):
             load()
             load.show() ## This will slow the thing down quite a lot
         self.bgd=Background2D(_data, self.boxsize).background
+        #fp.close()
         return self.bgd
 
     def calc_background(self,data, axis=None, masked=None):
-        if not self.bgd: self.__call__(data)
+        if self.bgd is None: self.__call__(data)
         return self.bgd
 
 
@@ -316,11 +354,7 @@ class PSFPhot_Routine(IterativelySubtractedPSFPhotometry):
                  background=None, wcs=None, verbose=1):
 
         group_maker=_grouping(crit_separation=crit_separation)
-
-        #self.background=background
-        bkg_estimator=BackGround_Estimate_Routine(None)
-        bkg_estimator.bgd=background
-
+        bkg_estimator=BackGround_Estimate_Routine(None, bgd=background)
         finder=Detection_Routine(sig_src=sig_src, sig_sky=sig_sky, fwhm=fwhm,
                 sharplo=sharplo, sharphi=sharphi, roundlo=roundlo, roundhi=roundhi,
                 wcs=wcs)
@@ -329,6 +363,7 @@ class PSFPhot_Routine(IterativelySubtractedPSFPhotometry):
         super().__init__(group_maker=group_maker, bkg_estimator=bkg_estimator,
                 psf_model=psf_model, fitshape=fitshape,
                 finder=finder, fitter=fitter, niters=1)
+        print("\x1b[33mWARNING: THIS IS UNDER DEVELOPMENT\x1b[0m")
 
     def _bkg(self, axis=None,masked=None):
         return self.background
@@ -340,12 +375,14 @@ class PSFPhot_Routine(IterativelySubtractedPSFPhotometry):
             init_guesses.rename_column("ycentroid", "y_0")
         """ 
         #if init_guesses: self.bkg_estimator.load_sources(init_guesses)
-        cat=super().do_photometry(image, init_guesses) 
+        cat=super().do_photometry(image, init_guesses)
 
-        cat.remove_columns(("flux_0", "group_id", "x_fit", "y_fit"))
+        #cat.remove_columns(("flux_0", "group_id", "x_fit", "y_fit"))
         cat.remove_rows( cat["flux_fit"]<=0)
-        cat.add_column(-2.5*np.log10(cat["flux_fit"]), name="mag_fit")
-        cat.add_column((2.5/np.log(10))*(cat["flux_unc"]/cat["flux_fit"]), name="mag_unc")
+        ###mag,magerr=flux2ABmag(
+        print("DO MAGCONV PROPERLY")
+        #cat.add_column(-2.5*np.log10(cat["flux_fit"]), name="mag_fit")
+        #cat.add_column((2.5/np.log(10))*(cat["flux_unc"]/cat["flux_fit"]), name="mag_unc")
         return cat
 
 class Cleaning_Routine(object):
@@ -437,6 +474,8 @@ class ArtificialStar_Routine(object):
         self.psffitter=psffitter
         self.psf=psf
 
+        print("WARNING: THIS IS UNDER DEVELOPMENT")
+
 
     def run(self, image, ntests=1000, subimage_size=500,  sources=None, fwhm=1,
             flux_range=(0,1e5), separation_thresh=2, save_progress=1):
