@@ -2,13 +2,16 @@ import time
 import os, sys, numpy as np
 from parse import parse
 from scipy.optimize import minimize
-from astropy.table import Table,hstack
+from astropy.table import Table,hstack,Column
 from astropy.io import fits
 import starbug2
+import requests
 
 printf=sys.stdout.write
 perror=sys.stderr.write
 puts=lambda s:printf("%s\n"%s)
+warn=lambda s:perror("\x1b[33m%s\x1b[0m"%s)
+sbold=lambda s:"\x1b[1m%s\x1b[0m"%s
 
 def strnktn(s,n,c):
     for _ in range(n): s+=c
@@ -70,24 +73,39 @@ def tabppend(base, tab):
         for line in tab: base.add_row(line)
     return base
 
-def export_region(tab, fname="/tmp/out.reg"):
+def export_region(tab, colour="green", scale_radius=1, region_radius=3, xcol="RA", ycol="DEC", wcs=1, fname="/tmp/out.reg"):
     """
-    A handy function to convert the detections in a DS( region file
+    A handy function to convert the detections in a DS9 region file
     """
-    xcol="xcentroid" if "xcentroid" in tab.colnames else "x_0"
-    ycol="ycentroid" if "ycentroid" in tab.colnames else "y_0"
-    xcol="RA"
-    ycol="DEC"
+
+    if xcol not in tab.colnames:
+        xcols= list(filter(lambda s: 'x'==s[0],tab.colnames))
+        if xcols:
+            xcol=xcols[0]
+            printf("Using '%s' as x position column\n"%sbold(xcol))
+            wcs=0
+
+    if ycol not in tab.colnames:
+        ycols= list(filter(lambda s: 'y'==s[0],tab.colnames))
+        if ycols:
+            ycol=ycols[0]
+            printf("Using '%s' as y position column\n"%sbold(ycol))
+            wcs=0
+
     
-    if "peak" in tab.colnames: r= np.sqrt(tab["peak"]**0.7)*2.5 
-    else: r=np.ones(len(tab))*2
+    if "flux" in tab.colnames and scale_radius: 
+        r= (-40.0/np.log10(tab["flux"]))
+        r[r<region_radius]=region_radius
+    else: r=np.ones(len(tab))*region_radius
+
+    prefix="fk5;" if wcs else ""
 
     with open(fname, 'w') as fp:
-        fp.write("global color=cyan\n")
+        fp.write("global color=%s width=2\n"%(colour))
         if tab:
             for src, ri in zip(tab,r[r>0]):
                 #fp.write("circle %f %f %f;"%(1+src[xcol], 1+src[ycol], ri))
-                fp.write("fk5;circle %f %f %fi\n"%(src[xcol], src[ycol], ri))
+                fp.write("%scircle %f %f %fi\n"%(prefix,src[xcol], src[ycol], ri))
         else:
             perror("unable to open %f\n"%fname)
 
@@ -145,7 +163,7 @@ def tab2array(tab,colnames=None):
 def export_table(table, fname=None, header=None):
     if table:
         if not fname: fname="/tmp/starbug.fits"
-        fits.BinTableHDU(data=table, header=header).writeto(fname, overwrite=True)
+        fits.BinTableHDU(data=reindex(table), header=header).writeto(fname, overwrite=True)
 
 def find_colnames(tab, basename):
     """
@@ -172,6 +190,7 @@ def combine_fnames(fnames, ntrys=10):
             fname+="(%s)"%"".join(sorted(set(chars)))
             trys+=1
         if trys>ntrys: return None
+    while ")(" in fname: fname=fname.replace(")(","")
     return "%s/%s%s"%(dname,fname,ext)
 
 
@@ -202,8 +221,9 @@ def hcascade(tables, colnames=None):
                 tab[cols[n]][mask]*=np.nan
                 if sum(mask): move=1
 
-        empty= ( np.nansum( tab2array( tab[cols] ),axis=0 ) ==0)
-        if any(empty): tab.remove_columns(np.array(cols)[empty])
+        if name != "flag": ## this is a bodge because it was removing the column if all the stars were good
+            empty= ( np.nansum( tab2array( tab[cols] ),axis=0 ) ==0)
+            if any(empty): tab.remove_columns(np.array(cols)[empty])
 
         cols=find_colnames(tab,name)#[ colname for colname in tab.colnames if name in colname]
         tab.rename_columns(cols, ["%s_%d"%(name,i+1) for i in range(len(cols))])
@@ -242,6 +262,26 @@ def flux2ABmag(flux,fluxerr=None, zp=1, filter=None):
     return flux2mag( flux, fluxerr, zp=zp)
 
 
+def wget(address, fname=None):
+    """
+    A really simple "implementation" of wget
+    """
+    r=requests.get(address)
+    if r.status_code==200:
+        fname=fname if fname else os.path.basename(address)
+        with open(fname,"wb") as fp:
+            for chunk in r.iter_content(chunk_size=128):
+                fp.write(chunk)
+    else: perror("Unable to download \"%s\"\n"%address)
+
+def reindex(table):
+    """
+    Add indexes into a table
+    """
+    if "Catalogue Number" in table.colnames: table.remove_column("Catalogue Number")
+    column=Column(["CN%d"%i for i in range(len(table))], name="Catalogue Number")
+    table.add_column(column,index=0)
+
 if __name__ == "__main__":
     import glob
     fnames=glob.glob("/home/conor/dat/NGC346/JWST/stage2-destriped/de-striped_F335M/de-striped_jw01227002001_02101_0000*_nrc*long_cal.fits")
